#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Scoary - Microbial Pan-GWAS GUI
By Ola Brynildsrud
Norwegian Institute of Public Health
olbb@fhi.no
"""

import sys, os
import threading

try:
    import Tkinter
except ImportError:
    #Python 3 issues
    try:
        import tkinter as Tkinter
    except:
        sys.exit("Need to have Tkinter / tkinter installed")
    
try:
    import tkFileDialog
except ImportError:
    # Python 3 issues
    try:
        from tkinter import filedialog
        tkFileDialog = filedialog
    except:
        sys.exit("Could not find tkFileDialog / filedialog")

try:
    import ttk
except ImportError:
    # Python 3 issues
    try:
        from tkinter import ttk as ttk
    except:
        sys.exit("Could not find ttk / tkinter.ttk")
    
try:
    ttk
    Tkinter
except NameError:
    sys.exit("Need the following installed: Tkinter, tkFileDialog, ttk")
    
try:
    import os
except ImportError:
    sys.exit("Need to be able to import os")
    
try:
    import argparse
except ImportError:
    sys.exit("Need to be able to import argparse")
    
try:
    import scoary.methods as sm
    import scoary
except ImportError:
    sys.exit("Could not find the main Scoary executable") 

from pkg_resources import resource_string, resource_filename

class ScoaryGUI(Tkinter.Tk):
    """
    Create the main GUI window
    """
    def __init__(self,parent):
        Tkinter.Tk.__init__(self,parent)
        self.parent = parent

        self.Scoary_parameters = {"GPA": None, "Trait":None,"Tree":None,"Restrict":None,"Writetree": False, "Delimiter":",", 
                                "Startcol": "15", "Maxhits":None,"Notime":False, "Outdir":None, "Permutations":0,
                                "Cutoffs": {"I": [1,0.05], "B": [0,1.0], "BH": [0,1.0], "PW": [0,1.0], "EPW": [1,0.05], "P":[0,1.0]},                                
                                }
        self.initialize_menu()
        
        self.toppart = Tkinter.Frame(self,height="350",width="800")
        self.bottompart = Tkinter.Frame(self,height="50",width="800")
        self.toppart.pack(side='top',expand=False)
        self.bottompart.pack(side='bottom',expand=True,fill='both')
        
        self.nwpart = Tkinter.Frame(self.toppart,height="350",width="250")
        self.nepart = Tkinter.LabelFrame(self.toppart,height="350",width="550",text="Control panel")
        self.nwpart.pack(side='left',expand=False)
        self.nepart.pack(side='right',expand=True)
        
        # Add further frames to top or bottom
        
        self.logocanvas = Tkinter.Canvas(self.nwpart,height="250",width="250",relief="ridge",bd=0,highlightthickness=0)
        self.logocanvas.pack(side='top',expand=False)
        
        self.citationframe = Tkinter.Frame(self.nwpart, height="100",width="250")
        self.citationframe.pack(side='bottom',expand=False,fill='none')
        
        self.initialize_controlboard()
        self.initialize_logo()
        self.initialize_citation()
        self.initialize_statusframe()
        
        ######## MENUS ########
        
    def initialize_menu(self):
        """
        Initialize the menu at the top
        """
        self.menubar = Tkinter.Menu(self,relief="flat")
        filemenu = Tkinter.Menu(self.menubar,tearoff=0)
        filemenu.add_command(label="About",command=self.AboutScoary)
        filemenu.add_separator()
        filemenu.add_command(label="Quit",command=self.quit)
        self.filemenu = filemenu
        self.menubar.add_cascade(label="File",menu=self.filemenu)
        
        optsmenu = Tkinter.Menu(self.menubar,tearoff=0)
        optsmenu.add_command(label="Clear", command=self.ClearAll)
        optsmenu.add_command(label="Test example", command=self.TestExample)
        self.optsmenu = optsmenu
        self.menubar.add_cascade(label="Options",menu=self.optsmenu)
        
        self.config(menu=self.menubar)        
        
    def initialize_citation(self):
        """
        Initialize the citation frame - below the logo
        """
        myfontstyle = ("Arial",8)
        self.citation = Tkinter.Label(self.citationframe, text=self.citationtext(),anchor='center',justify='center', font=myfontstyle)
        self.citation.pack(expand=True)
        
    def initialize_statusframe(self):
        """
        Initialize the frame and statusbar occupying the bottom
        """
        frame = self.bottompart
        
        frame.pb = ttk.Progressbar(frame,orient='horizontal',mode='determinate',maximum=100)
        frame.pb.pack(fill='both',expand=True,side='top')

        frame.lab = Tkinter.Label(frame,text=u"Awaiting input options")
        frame.lab.pack(in_=frame.pb,expand=True)
        sys.stdout = StdoutToLabel(frame.lab, progressbar=frame.pb)
    
    def initialize_controlboard(self):
        """
        Initialize the controlboard - where all the settings are set.
        """
        board = self.nepart
        
        self.GPAentryVariable = Tkinter.StringVar()
        board.GPAentry = Tkinter.Entry(board, textvariable = self.GPAentryVariable,width=60)
        board.GPAentry.grid(column=0,row=0,sticky='W',columnspan=2)
        self.GPAentryVariable.set("Path to gene presence absence file")
        
        self.TraitsentryVariable = Tkinter.StringVar()
        board.Traitsentry = Tkinter.Entry(board, textvariable=self.TraitsentryVariable,width=60)
        board.Traitsentry.grid(column=0,row=1,sticky='W',columnspan=2)
        self.TraitsentryVariable.set("Path to traits/phenotype file")
        
        self.TreeentryVariable = Tkinter.StringVar()
        board.Treeentry = Tkinter.Entry(board, textvariable=self.TreeentryVariable,width=60)
        board.Treeentry.grid(column=0,row=2,sticky='W',columnspan=2)
        self.TreeentryVariable.set("(Optional) Path to custom tree file")
        
        self.RestrictVariable = Tkinter.StringVar()
        board.Restrictentry = Tkinter.Entry(board, textvariable=self.RestrictVariable,width=60)
        board.Restrictentry.grid(column=0,row=3,sticky='W',columnspan=2)
        self.RestrictVariable.set("(Optional) Path to file naming isolates to include")
        
        self.Outputdir = Tkinter.StringVar()
        board.Outputentry = Tkinter.Entry(board, textvariable=self.Outputdir,width=60)
        board.Outputentry.grid(column=0,row=4,sticky='W',columnspan=2)
        self.Outputdir.set("(Optional) Output directory")

        browsebuttonGPA = Tkinter.Button(board, text=u"Browse...", command=self.BrowseButtonClickGPA)
        browsebuttonGPA.grid(column=2,row=0,sticky='e')
        
        browsebuttonTraits = Tkinter.Button(board,text=u"Browse...", command=self.BrowseButtonClickTraits)
        browsebuttonTraits.grid(column=2,row=1,sticky='e')
        
        browsebuttonTreeFile = Tkinter.Button(board,text=u"Browse...", command=self.BrowseButtonClickTreeFile)
        browsebuttonTreeFile.grid(column=2,row=2,sticky='e')
        
        browsebuttonRestrict = Tkinter.Button(board,text=u"Browse...",command=self.BrowseButtonClickRestrict)
        browsebuttonRestrict.grid(column=2,row=3,sticky='e')
        
        browsebuttonOutput = Tkinter.Button(board,text=u"Browse...",command=self.BrowseButtonClickOutput)
        browsebuttonOutput.grid(column=2,row=4,sticky='e')
        
        # Initialize frame for cutoffs
        board.pframe = Tkinter.LabelFrame(board,text="Cut-offs",relief='ridge')
        board.pframe.grid(column=0,row=5,sticky='w')
        self.initialize_pvalueframe()
        
        # Initialize frame for misc options
        board.miscframe = Tkinter.LabelFrame(board,text="Misc options",relief='ridge')
        board.miscframe.grid(column=1,row=5,sticky='e',columnspan=2)
        self.initialize_miscopts()
        
        ## Create extra space
        
        board.emptyspace = Tkinter.Frame(board)
        board.emptyspace.grid(column=0,row=6,columnspan=3,pady=(20,20))
        
        masterbuttonfont = ("Courier", 16)
        
        RunButton = Tkinter.Button(board.emptyspace,text=u"Run analysis",font=masterbuttonfont,command=self.RunAnalysis,padx=15,pady=15)
        RunButton.grid(column=2,row=0)
        
        QuitButton = Tkinter.Button(board.emptyspace,text=u"Quit",font=masterbuttonfont, command=self.quit,padx=15,pady=15)
        QuitButton.grid(column=1,row=0)
        
        HelpButton = Tkinter.Button(board.emptyspace,text=u"Help",font=masterbuttonfont,command=self.HelpButton,padx=15,pady=15)
        HelpButton.grid(column=0,row=0)
        
    def initialize_miscopts(self):
        """
        Initialize the miscellaneous options
        """
        board = self.nepart
        miscframe = board.miscframe
        
        # Max hits
        self.mhtext = Tkinter.StringVar()
        miscframe.mhlab = Tkinter.Label(miscframe,textvariable=self.mhtext)
        miscframe.mhlab.grid(column=0,row=0,sticky='w')
        self.mhtext.set("Max hits")
        
        self.maxhitsvar = Tkinter.StringVar()
        miscframe.maxhitsentry = Tkinter.Entry(miscframe,textvariable=self.maxhitsvar,width=8)
        miscframe.maxhitsentry.grid(column=1,row=0)
        self.maxhitsvar.set("")
        
        # Delimiter
        self.delimtext = Tkinter.StringVar()
        miscframe.delim = Tkinter.Label(miscframe,textvariable=self.delimtext)
        miscframe.delim.grid(column=0,row=1,sticky='w')
        self.delimtext.set("Delimiter")
        
        self.delimvar=Tkinter.StringVar()
        miscframe.delimentry=Tkinter.Entry(miscframe,textvariable=self.delimvar,width=8)
        miscframe.delimentry.grid(column=1,row=1)
        self.delimvar.set(",")
        
        # Starting column
        self.sctext = Tkinter.StringVar()
        miscframe.sclab = Tkinter.Label(miscframe,textvariable=self.sctext)
        miscframe.sclab.grid(row=2,column=0,sticky='w')
        self.sctext.set("Startcol GPA file")
        
        self.scvar = Tkinter.StringVar()
        miscframe.sc = Tkinter.Entry(miscframe,textvariable=self.scvar,width=8)
        miscframe.sc.grid(row=2,column=1)
        self.scvar.set("15")
        
        # Permutations
        self.permtext = Tkinter.StringVar()
        miscframe.permlab = Tkinter.Label(miscframe,textvariable=self.permtext)
        miscframe.permlab.grid(row=3,column=0,sticky='w')
        self.permtext.set("Permutations")
        
        self.permvar = Tkinter.StringVar()
        miscframe.perm = Tkinter.Entry(miscframe,textvariable=self.permvar,width=8)
        miscframe.perm.grid(row=3,column=1)
        self.permvar.set("0")
        
        # No timestamp
        self.notimevar = Tkinter.IntVar()
        miscframe.notime = Tkinter.Checkbutton(miscframe,text=u"No timestamp",onvalue=1,offvalue=0,variable=self.notimevar)
        miscframe.notime.grid(row=5,column=0,sticky='w')
        
        # Write tree
        self.writetreevar = Tkinter.IntVar()
        miscframe.writetree = Tkinter.Checkbutton(miscframe,text=u"Write tree",onvalue=1,offvalue=0,variable=self.writetreevar)
        miscframe.writetree.grid(row=6,column=0,sticky='w')
        
    def initialize_pvalueframe(self):
        """
        Initialize the filtration controlboard
        """
        board = self.nepart
        pframe = board.pframe
        
        # Add p-value checkboxes
        
        self.pVar = Tkinter.IntVar()
        self.pVar.set(1)
        self.pBVar = Tkinter.IntVar()
        self.pBHVar = Tkinter.IntVar()
        self.pPWVar = Tkinter.IntVar()
        self.pEPWVar = Tkinter.IntVar()
        self.pEPWVar.set(1)
        self.pPermVar = Tkinter.IntVar()
        
        pframe.pNcheck = Tkinter.Checkbutton(pframe,text=u"Naive (Fisher's)",onvalue=1,offvalue=0, variable=self.pVar)
        pframe.pBcheck = Tkinter.Checkbutton(pframe,text=u"Bonferroni",onvalue=1,offvalue=0, variable=self.pBVar)
        pframe.pBHcheck = Tkinter.Checkbutton(pframe,text=u"Benjamini-Hochberg",onvalue=1,offvalue=0, variable=self.pBHVar)
        pframe.pPWcheck = Tkinter.Checkbutton(pframe,text=u"Pairwise comparison (Best)",onvalue=1,offvalue=0, variable=self.pPWVar)
        pframe.pEPWcheck = Tkinter.Checkbutton(pframe,text=u"Pairwise comparison (Entire)",onvalue=1,offvalue=0, variable=self.pEPWVar)
        pframe.pPermcheck = Tkinter.Checkbutton(pframe,text=u"Empirical p-value (Permutation)", onvalue=1,offvalue=0, variable=self.pPermVar)
        
        pframe.pNcheck.grid(column=0,row=0,sticky='w')
        pframe.pBcheck.grid(column=0,row=1,sticky='w')
        pframe.pBHcheck.grid(column=0,row=2,sticky='w')
        pframe.pPWcheck.grid(column=0,row=3,sticky='w')
        pframe.pEPWcheck.grid(column=0,row=4,sticky='w')
        pframe.pPermcheck.grid(column=0,row=5,sticky='w')
        
        # Add p-value entry cells
        
        self.pNaive = Tkinter.StringVar()
        self.pBonf = Tkinter.StringVar()
        self.pBH = Tkinter.StringVar()
        self.pPW = Tkinter.StringVar()
        self.pEPW = Tkinter.StringVar()
        self.pPerm = Tkinter.StringVar()
        
        pframe.pNaiveEntry = Tkinter.Entry(pframe,textvariable=self.pNaive,width=8)
        pframe.pBonfEntry = Tkinter.Entry(pframe,textvariable=self.pBonf,width=8)
        pframe.pBHEntry = Tkinter.Entry(pframe,textvariable=self.pBH,width=8)
        pframe.pPWEntry = Tkinter.Entry(pframe,textvariable=self.pPW,width=8)
        pframe.pEPWEntry = Tkinter.Entry(pframe,textvariable=self.pEPW,width=8)
        pframe.pPermEntry = Tkinter.Entry(pframe,textvariable=self.pPerm,width=8)
        
        pframe.pNaiveEntry.grid(column=1,row=0,sticky='w')
        pframe.pBonfEntry.grid(column=1,row=1,sticky='w')
        pframe.pBHEntry.grid(column=1,row=2,sticky='w')
        pframe.pPWEntry.grid(column=1,row=3,sticky='w')
        pframe.pEPWEntry.grid(column=1,row=4,sticky='w')
        pframe.pPermEntry.grid(column=1,row=5,sticky='w')
        
        self.pNaive.set("0.05")
        self.pBonf.set("1.0")
        self.pBH.set("1.0")
        self.pPW.set("1.0")
        self.pEPW.set("0.05")
        self.pPerm.set("1.0")
        
        ######## EVENTS ########
        
    def AboutScoary(self):
        """
        Placeholder button. Planned short information about the method
        """
        topwin = Tkinter.Toplevel(self)
        button = Tkinter.Button(topwin,text=str("https://github.com/AdmiralenOla/Scoary"))
        button.pack()
        
    def BrowseButtonClickGPA(self):
        """
        Browse button for gene presence absence field
        """
        myfile = tkFileDialog.askopenfilename(filetypes=[('comma-separated values', '.csv'), ('all files','.*')])
        self.GPAentryVariable.set(myfile)
        
    def BrowseButtonClickTraits(self):
        """
        Browse button for traits field
        """
        myfile = tkFileDialog.askopenfilename(filetypes=[('comma-separated values', '.csv'), ('all files','.*')])
        self.TraitsentryVariable.set(myfile)
        
    def BrowseButtonClickTreeFile(self):
        """
        Browse button for tree field
        """
        myfile = tkFileDialog.askopenfilename(filetypes=[('newick tree files', '.nwk'), ('all files','.*')])
        self.TreeentryVariable.set(myfile)
        
    def BrowseButtonClickRestrict(self):
        """
        Browse button for isolate restriction field
        """
        myfile = tkFileDialog.askopenfilename(filetypes=[('comma-separated values','.csv'),('all files','.*')])
        self.RestrictVariable.set(myfile)            
        
    def BrowseButtonClickOutput(self):
        """
        Browse button for choosing output dir
        """
        mydir = tkFileDialog.askdirectory(mustexist=True)
        self.Outputdir.set(mydir)
    
    def HelpButton(self):
        """
        Placeholder button. Redirects to website
        """
        print("Visit https://github.com/AdmiralenOla/Scoary for help")
        
    def ClearAll(self):
        """
        Sets all variables to defaults
        """
        self.GPAentryVariable.set("Path to gene presence absence file")
        self.TraitsentryVariable.set("Path to traits/phenotype file")
        self.TreeentryVariable.set("(Optional) Path to custom tree file")
        self.RestrictVariable.set("(Optional) Path to file naming isolates to include")
        self.Outputdir.set("")
        self.maxhitsvar.set("")
        self.delimvar.set(",")
        self.scvar.set("15")
        self.notimevar.set(0)
        self.writetreevar.set(0)
        self.permvar.set("0")
        self.pVar.set(1)
        self.pBVar.set(0)
        self.pBHVar.set(0)
        self.pPWVar.set(0)
        self.pEPWVar.set(1)
        self.pNaive.set("0.05")
        self.pBonf.set("1.0")
        self.pBH.set("1.0")
        self.pPW.set("1.0")
        self.pEPW.set("0.05")
        self.pPerm.set("1.0")
        
    def TestExample(self):
        """
        Sets all variables corresponding to --test in the methods script
        """
        self.GPAentryVariable.set(str(os.path.join(resource_filename(__name__, 'exampledata'), 'Gene_presence_absence.csv')))
        self.TraitsentryVariable.set(str(os.path.join(resource_filename(__name__, 'exampledata'), 'Tetracycline_resistance.csv')))
        self.TreeentryVariable.set("")
        self.RestrictVariable.set("")
        self.Outputdir.set("./")
        self.maxhitsvar.set("")
        self.delimvar.set(",")
        self.scvar.set("15")
        self.notimevar.set(0)
        self.writetreevar.set(0)
        self.permvar.set("0")
        self.pVar.set(1)
        self.pBVar.set(0)
        self.pBHVar.set(0)
        self.pPWVar.set(0)
        self.pEPWVar.set(1)
        self.pNaive.set("0.05")
        self.pBonf.set("1.0")
        self.pBH.set("1.0")
        self.pPW.set("1.0")
        self.pEPW.set("0.05")
        self.pPerm.set("1.0")
        
        ######## RUNNING THE ANALYSIS ########
        
    def RunAnalysis(self):
        """
        Upon click "Run analysis" - Reads all the set parameters and calls run method
        """
        self.Scoary_parameters["GPA"] = self.GPAentryVariable.get()
        self.Scoary_parameters["Traits"] = self.TraitsentryVariable.get()
        self.Scoary_parameters["Tree"] = self.TreeentryVariable.get()
        self.Scoary_parameters["Outdir"] = self.Outputdir.get()
        self.Scoary_parameters["Restrict"] = self.RestrictVariable.get()
        self.Scoary_parameters["Writetree"] = self.writetreevar.get()
        self.Scoary_parameters["Delimiter"] = self.delimvar.get()
        self.Scoary_parameters["Startcol"] = self.scvar.get()
        self.Scoary_parameters["Maxhits"] = self.maxhitsvar.get()
        self.Scoary_parameters["Notime"] = self.notimevar.get()
        self.Scoary_parameters["Permutations"] = self.permvar.get()
        self.Scoary_parameters["Cutoffs"]["I"][0] = self.pVar.get()
        self.Scoary_parameters["Cutoffs"]["I"][1] = self.pNaive.get()
        self.Scoary_parameters["Cutoffs"]["B"][0] = self.pBVar.get()
        self.Scoary_parameters["Cutoffs"]["B"][1] = self.pBonf.get()
        self.Scoary_parameters["Cutoffs"]["BH"][0] = self.pBHVar.get()
        self.Scoary_parameters["Cutoffs"]["BH"][1] = self.pBH.get()
        self.Scoary_parameters["Cutoffs"]["PW"][0] = self.pPWVar.get()
        self.Scoary_parameters["Cutoffs"]["PW"][1] = self.pPW.get()
        self.Scoary_parameters["Cutoffs"]["EPW"][0] = self.pEPWVar.get()
        self.Scoary_parameters["Cutoffs"]["EPW"][1] = self.pEPW.get()
        self.Scoary_parameters["Cutoffs"]["P"][0] = self.pPermVar.get()
        self.Scoary_parameters["Cutoffs"]["P"][1] = self.pPerm.get()
        
        self.PrepareScoaryCMDline()
        
    def PrepareScoaryCMDline(self):
        """
        Prepares arguments to correspond with argparse namespace and runs.
        Listens to sys.stdout and updates statusbar while scoary runs
        """
        citation=False
        RunScoary = True
        correction = []
        p_value_cutoff = []
        for m in self.Scoary_parameters["Cutoffs"]:
            if self.Scoary_parameters["Cutoffs"][m][0] == 1:
                correction.append(m)
                try:
                    p_value_cutoff.append(float(self.Scoary_parameters["Cutoffs"][m][1]))
                except ValueError:
                    print("Please enter real numbers in the p value fields")
                    RunScoary = False
                    break
                    
        delimiter = self.Scoary_parameters["Delimiter"]
        genes = self.Scoary_parameters["GPA"] if self.Scoary_parameters["GPA"] not in ["","Path to gene presence absence file"] else None
        try:
            max_hits = int(self.Scoary_parameters["Maxhits"]) if self.Scoary_parameters["Maxhits"] not in [""] else None
        except ValueError:
            print("Please enter a real number (or nothing) in the max hits field")
            max_hits = None
            RunScoary = False
        try:    
            permutations = abs(int(self.Scoary_parameters["Permutations"])) if self.Scoary_parameters["Permutations"] not in [""] else 0
        except ValueError:
            print("Please enter a real number (or nothing) in the max hits field")
            permutations = 0
            RunScoary = False
        newicktree = self.Scoary_parameters["Tree"] if self.Scoary_parameters["Tree"] not in ["", "(Optional) Path to custom tree file"] else None
        no_time = True if self.Scoary_parameters["Notime"] == 1 else False
        outdir = self.Scoary_parameters["Outdir"] if self.Scoary_parameters["Outdir"] not in ["","(Optional) Output directory"] else "./"
        restrict_to = self.Scoary_parameters["Restrict"] if self.Scoary_parameters["Restrict"] not in ["", "(Optional) Path to file naming isolates to include"] else None
        start_col = self.Scoary_parameters["Startcol"]
        test = False
        threads = 1
        traits = self.Scoary_parameters["Traits"] if self.Scoary_parameters["Traits"] not in ["","Path to traits/phenotype file"] else None
        upgma_tree = True if self.Scoary_parameters["Writetree"] == 1 else False
        write_reduced=False
        collapse=False
               
        myargs = argparse.Namespace(citation=citation, correction=correction, p_value_cutoff=p_value_cutoff, delimiter=delimiter, genes=genes, max_hits=max_hits, newicktree=newicktree,no_time=no_time,restrict_to=restrict_to,
        outdir=outdir,permute=permutations,start_col=start_col,test=test,threads=threads,traits=traits,upgma_tree=upgma_tree,write_reduced=write_reduced,collapse=collapse)
        
        if RunScoary:
            try:
                
                sm.main(args=myargs, cutoffs=dict(list(zip(correction, p_value_cutoff))),statusbar=sys.stdout)

            except SystemExit as SE:
                # Set status bar color to red?
                if str(SE) == "0":
                    print("Analysis complete!")
                else:
                    print("Fatal error: %s" % str(SE))
        
        # Listen to stdout and update statusbar
        
    ######## MISC METHODS ########
        
    def initialize_logo(self):
        """
        Initialize logo
        """
        photo=Tkinter.PhotoImage(data=self.Photobase64())
        self.logocanvas.img = photo
        self.logocanvas.create_image(0,0,anchor='nw',image=photo)
        
    def citationtext(self):
        """
        Returns citation info
        """
        text = "SCOARY version %s \n\n" \
        "Please cite as: \n" \
        "Brynildsrud O, Bohlin J, Scheffer L, \n" \
        "Eldholm V. Rapid scoring of genes in \n" \
        "microbial pan-genome-wide association \n" \
        "studies with Scoary. Genome Biol. \n" \
        "2016;17:238" % scoary.__version__
        return text
        
    def Photobase64(self):
        """
        base64 encoding of the logo
        """
        photo= """
        R0lGODlh+gD6AOf/AAABAAEEAAMGAgcJBQoMCAwPCw8RDhIUERYYFhkbGRwdGx8gHiIkISYoJSstKjsrLkArL0gqJ00pI0QsJzEyMEQtLT8wMkYvL0IxLkUwM0svMDQ2M0IyNE0wMUQzMEMzNUoyMjY4NUQ1N0Y1\
        MkI3ODk7OEc3OUk5PEU7PD4/PU07OUU9QkE/Q04+QD9DRUFDQERETVBBSEtDSE1DQ0dFSEZHRVRDRlZDQUFJUUlLSD1PWVRKSjhRWkxOSzRVY1xMUEdSXj5VZFBST0BWYFNVUjRcbl9VWFZYVWJVUT9ebEtdaDZj\
        dVtdWkBjd2lcVyNsh2hcXSxrgV9hXjlsg2VnZDV0inFmZzF3kmpsaXZpZyl8lh5/nit9nm5wbSeEo3hzcjiCnXN1ch6JroBzcnZ4dYF2di+KqRaQuoB4cnh6dyeOs4d5dBiUuASaw3x+e4CBf4mBeyyXwACj0wCh\
        8YyFgIWHhJCEhBugygCm8ACo8QCp8h6lzhGo2ACr7h+i7QCr9IuNigCu4wCw3gqt8I6QjRyr2wCy9BCu8QC16ZqSjJKUkQC28Q2z6QC2+Bew8wC59AC67wC6/Be166GWip6XkQC895eZlgC9/iC18qSclSa49QjB\
        9qKdnJ2fnBDA+yi78aWgny2+9KKkoTLA9qWnpKynpamrqEfD9Kyuq7OtqjvJ+EvG97iysbK0sE7M9lvK9be5tl/N+bu9umDS9768wMS8tWzQ9sDCv8bBwHvT+3rW94XW+MbIxM/HwHza+4fY+snLyM7KzIjd95Hb\
        +M3PzJXf/J3e/NfSytjS0dPV0ZHl+aXg+drWx57j+tXX1Kjj/N3a0dnb2Kjn+bLo+7nn++Te2t7h3ePi2bvu+8Ps+uPl4ubm3Mbv/c3t/u7o5+jq5svy+uzr4tTx++3s4+zu6/Pt7Nj0/+7w7d7z//Hw5t32++T1\
        /PL08Pjz8ff27ej5//T38/D4/+z6+/P4++77/PX6/fj69/r8+ff9//n+///9+/z++//9////9v7//P///yH+C1Njb2FyeSBsb2dvACH5BAEKAP8ALAAAAAD6APoAAAj+AP0JHEiwoMGDCBMqXMiwocOHECNKnEix\
        osWLGDNq3Mixo8ePIEOKHEmypMmTKFOqXMmypcuXMGPKnEmzps2bOHPq3Mmzp8+fQIMKHUq0qNGjSJMqXcq0qdOnUKNKnUq1qtWrWLNq3cq1q9evYMOKHUu2rNmzaNOqXcu2rdu3cOPKnUu3rt27ePPq3cu3r9+/\
        gAMLHky4sOHDiBMrXsy4sePHkCNLnky5suXLmDNr3sy5s+fPoEOLHk26tOnTqFOrXs26tevXsGPLnk27tu3buHPr3s27t+/fwIMLH068uPHjyF2fs5asebRv9pKTZJcsDIMA2LNrT0FIGzzpHJP+MRigvbz5AANy\
        sAN/EZiD8/DP17jHfuKb+PjPt6oPkUj+/+URwl9D/gFoIHaADKgQIQc2GIA1CiJEnoMGDhBdhARhAeABCjDQQAMLHGAgKhgOxM5/adhDX0H21AFgiQIRgx8V6ynEzgv5yQKji/C98BAC+IUBYw3x1djQN/hRAGMD\
        8A3w3UMixudTO8yUYscaaHBSyzeNXQffkw4Bgt9EtwjBwAETYjeAAQykIIpGyJSxgwkcWFDBnRVAwEELUICS0TdhOKBAAeatqYADZHDpkD2iNOpoo6QQZM0GBmRXAANk+APLo5zq+BCjnD4K5kMlxFcHRCeqWQCH\
        HjZwzqf+dQBpIBXaVMQKEnp+4IEJW6hxhhpiPOGDCXeaYIeiEqGygYMhwNIQOVIKdF98C8R3AES8xEfARBrG5yxKt1CIXScSVWMEBLp64MEFUTyCybuLaBJJJW0EQacFqUTETgriYifEhQmdA9+2/qSRnyX4RfNQ\
        t+dFKpEs+flo0hH9YlcDRKVA4AEHHnzAwQcRMPLuyCRjEsgFHO8wqkLwEFAxdtcqJPB525rynzVMxHexQ/ENQJE2ADagyEilvhyAxAyVYYHH6m7sQRWNlFyyJyw03UI1DNlTqdEBMCDzwP6k2aQ/EDcJsEKwxOdA\
        RV4CGEIannJEMdcBYMHQGhyry/T+BxUEInXJl+igLscmQLMQFXRj9+1BM5tHwHv5kRt2fBAylDN8+/0srgN1EEMORgwmHoDCCSXCgQkngHABBhtzMIW7f5NcxMdNm1BOQsCIHoCSCDVO4QArhn4eEw3J6vhFOFas\
        gChnR/QfEbewYw8wl1ubUCpLM7KIIHFUAQQJHwgSe8lcYMDB+efP0PxAycNXQivn2HMOMG8Qit/6Avnu4AIDAR0f/gRJRnykgJFqGU0BQ4vIsuLDBCMNBB7VO0/cCJIOXfFhEfB6hAaDMATYjQ8TYrhA61qXiIPY\
        w1rIKkjZ4MOL3h2oB0domyUIYjzz3GIhRILPDS/yDZdxbQD+XQCgQaAVn0wlpGjnycFBoFCBKmBQaoHAgBw++C45VGCEG3vAMQyCOPhUDiE5iI/DDKK/8xRgcf74Rh18RpAVmsduComSeQqgEXIoQHSmaAiPzrO2\
        hYitPM2DBgdo4Inx6SAJlfjgInBAAx644AQXiCQHsmCQHjgAAX98w0KsER8BMe4/r2qI/c6jECKeJ0EbCWPikJYQ/HwxIQvMzgAWEIYdEuQHFVDDByURgSnGbhEskEEhNYiIO1QBBQ+4hkKIQQgmJOCVCImPJj+J\
        nxk6hF+YS0gXz0O6jWgDm1xrgEIEGJ8VKQQQC+gCLKxBDgB64wM28ODfHuEDGmxhD5D+8MQlNAEvGKzgiSNbxCO0UAFKauSP2TEiGfMDEXKehwgIuUd86PiRZKQBoRSCKEJsBh8hcAQUHvBC1Mb3CB1w4AIV6IAM\
        dLCEK6gBCCwAaMk+AYIWaEQU8RESNeFzhIi0rTwGQIhDzUMikciCCAbs15sOYjD4jDEjNsBAIajIB5R5DH3nqwALChm7SyTBAtugCHVIkcOcujA+E2yIG+LTQoPssTwWOgkpQoDR/LDRIKo8T600wo4HmECmUlsE\
        DAbntI95zAWJHN8WLJCJT8HDokn9j04XSrmI+O88IThIXb2WEnt8AxY9aFAaDgI5Um4kGBCQgTxLtgg+5I2weeP+AApWW7I4VMAIDDnHGxiwNQcptCBl1E4KHYLE8hSgebnLpkuIEav/cLYgNSwPR2phAQxo4RKx\
        ewQNnJau2J7Ub+MLxARkkJBoUCGy4hrtTs8TSoi4sTzdFAgZ4EPRmHSit/A5iA9Nq5FSnG8CW6AtJtqAAY/p6nwdO98HMNAGwJJsERH4wPq0IYTEqZeyX5JIquRTECY99CP3UNFE7hgfWw6krgHgSCkK7IEKcEGm\
        i7gEED5wVSxe9QJPEDC8IsABB/ojXA7ar3nccFb4mBMiESyPkZCkQ4xYAxaK6EIOGICAAxBgmhHhZHyIXBA5mkfFKHMaGC7hiUcgggtNuMD+gZsW26YBwcEB5bEDSdGgDXTic0LWDpaBO6aJBBc7bfXHNssTM4vk\
        1TyshAgF4tPTgvy0PHvNCCvQ57EL6IAFHZAADXwAAsMa9qoI/gAJ4PwuRkzABObccM+kQIpkFCTP2dkzQf6MnYp4WTsEFAiJzXOqiwjPPEGdiBTik2h/gNM8edQILljMMQ4sQQxyWIQ+2wACpxG2sB1jBD9Z+wgv\
        VCAGBInleYQQaIPUVdYDoXWKKdJU054QPq6+yFDNk7mIVBg+SizIoMuTa4ZYwnM+lpTGBjdqqe1BBE3bmK6YtisQsGERAF3EHVBgvi88ED9pNYipilwoi8Qnj9M6Xkb+8IOAicA6O1QwSCd6xjNVUaAO8R6IPR7Q\
        7AuYIXZ76PRVQX2BKkjCDCBYgRo0eIcVINgCtBiIIuKzVIVsfL3GtUj7ygPRj2ukrOcp6kNWLkaDZCs+n1sIKhhNECs4jQU6fkQbwtxsDWghEO56BCPAAAIc6MCq5xOBOwYi7vKEPSGqLg8qMWyeQk+EzucZQOBl\
        GXCKIAw/Mc/tKM+zvnfDZ/CwfLpAguGxCtx8fI2g9rpMEAVErFYTnwiCgvP2A4Lg58gHwSl8ek348pTc4zyFz3MxsnjzAONI+VEAQu7tRRvhJ4XsoBMMuPrBO3igCZAY6d/ioOaPcQACjR0Iftr+i5Bdmwfd+YvP\
        7SsSBgfVOyNd+E8IhOgPhg0QIVqGT7BNeHLslMAgdMhAFLb9QT54QGQfxAhs9wEn0A+ux1YK8XXwwWW1px3jRxEyciDzpxFM9h8NIAR1IAqmIAp1QASLBiBChF+OIzkEAQzeR28GYQ8qkAFTpUh3EAM6NjKecAKD\
        kwGUUBClZR4PqEL50QUcZ3sYUX9J5BFvpTvZcWEHEYH4cQBSAAiKQAY5aEYIQQkQgAMxiAmXEAVKcIWYUAk4gD4zAHtFWB4h8HcC8Q3plx/EA3UOiBGIByAmthEeZoTYQTAKgXVcE18FAQU9F4OXMARewH/ZpXoe\
        s0UFAQ/+/4EFvJAMqBBaAFJs4QcfOzgR7IBi2jGBHSGERgNNCBFdFeNRCREOD/ABXqBjlRABe0BqJDMFHoABk9B9dCh8bJgdkzgRjwYf4EeBltgvxOAQFfgyvKMQx/AAF3AF2EUyj7AIRWBzV9haK2ABY5AQl/U7\
        /zOL2FGLEqGAkCcS7PCBdHMAvfgQ5JAAL3N/DdELHFABS/BEjxAIT9Bp6SgGphhCDxCNCyIuFGB55uFJfCaJGuGJhFYSYbCLANIDFHFsDbKGDnEMLSBb0aYFEZBg6jIBQ1cycrcEIgQHeuQgoOiI5iFODUiLGpFk\
        5vFUI5GG4iIEKxMRyTB5/5EAw9X+EPAABQt3YDTWOhegS++iCY/QBDTGAb3wEEqYHwfwRRzFX7MmfhoRDfkRkyIBD7zQBbcGHwdABbfAfRZBDO53HgbgBs5gEbXQAg8wOAemYB5gBoXABTpgAhfwAHCAlQzBDqJw\
        gtpBBL9HEOSQA0Kwl3wpBD2QQATBDnrZl3zZbxgRhdrBPywBlaZACG4QBmFABm+gCKgADOx3EdbQCoSQBpD5BpYgC2Z4EazgBHVCaehTASj1ADdAB1hTEd+ACoRABmHgBpZQbjuBcYIxDpQABTegAiogAiOgAjfw\
        A2sQlF7xhnN0GPcAD+zQDtGhD2ERAvGRAq3BDcuwDPOQEtj+cJ3ZKZT4EYcvgQl40AcgQQ9zgAenkBDnmZ4MMQp4oAf4IBPvsAp6gAf2iQeGMAwDcZ8EsQr26QcEcQr2+Q4D8Qr2iQkHgQ8Gep94oAnc4BAiqB2G\
        FxOaoAeHABLzMJ6vkBB+oAfsuRCn0AeHEJ8wYQ7j2Qd/oAd60AcoqgsCEaJ4IA4DYQgsigfZMBCaIKIkig+DwKJ9QKIByqIpuqJ9oAcEqhD3cGjfhxUZ2gcbihEhOqIwgQ9/wKKYMA3+8A6bwKJ6cKPPkKL66Q/q\
        cKJ94KICsaIIKhDi4KN/EAsFMQ3jeQg3ig6YwKKaYCKh2Qq76JQhAQ7GIKMCYQ7GsA7+DDEPy9AMR2oQ2NAMBSGojEoQTbqh77AMgCoQQEoQ+PAMh1oQUXqpCboMN2oQ72AMy4ANCTGqoZoQv8Ciq1AQulCfruAP\
        88CqArELRNoH7Amnf2Cm/vAJLPoIKFoPBDEMVUqoA3EIeDAIJEpOBnAAU3keKTcS+IAHfqAMofCeeDAK73Ct9RkKJCqe5CkQ28qgj3Ck7okJwmCfd+oPzyCe9/mksjqeq+CfBwoOA7GeliqgDOqmL6qjBjGteLAL\
        tnCfhmCq4qoJ2DqezyAQ8eAH1Sqg9WkIxnoQNPoH6lAQ9PAMgIoPh9AHjyAQodAHmBCygyAQtvAHeCCs/rAOGjr+DCxKDQSxDCsaCuYwEOLQDd3QPxTCpxyBDyw6CIYQCjSKokFbp3+wCwIBrI4wEEb7CSHbB4bw\
        on9wCHrwB7vqD9lQpUELrH+Qpk1qoaHgq1C7n33Qqv4wCyjqtDSKB4/aqf+6oofwB6HwtD8qEDR6CHNbowLRpIZQtHaKENOKogxRpyXrD4bwB6OADfUpo8AatQLhsn+QDSbaB59QED6Kn7bQDSo7EKZkIJMlEj47\
        tv4gDuOpBzX7DizKnkorELpQpdIgENmAoox6ClXaB7YQCjdapYXrDyf7B8UQr30QowIxCymKtP4gr/4QuGbqs38Qq/7gtgWBDyuqB7/rD8X+ULVuSg1VCqjFgLJY2qRS6g816qkCAQ7jWbkDkQvqq76/YLIoiw4Z\
        igdYOgh6kAv4QKPo6w91igfx6at4ULMDwQ0+6qN6YAsEoY/OZRKh+6Hj6bj0wKKjkLR9sLT+MAr+mrzmsA7xSbt60L6wW6WPKhA+Crz5K74iKxDIa6kkug4s+qTQi6kr6rgC4QgsaqmbawzeC7yhMBAw2p0FYb4e\
        SxB4YLVWm6akqwe6sAspKhB1+rEo+qTZMJ7OG7u2WxDvsKpDLKQRrH0GwgCXuRHM67zHi6uW2qNbvLr6C7Xk+7wom6jLgKIPOhBPWw9Nyq9MrMZjbLb+gA22MAqGcKL+LnzBMNwHOyzHKEqiz/AKIbui8huvfwCv\
        qxC82bmg9lm5pUsQoTAKT5um/tCj7hLE/rAKf+AIWdvIocyi05AN2UANLCrDjdoMosyiQIqYw/PFYIyiYjye7MmjfXDGE3zH4cupKBuzcIzJLEoPkUoQdeq4KTywVYoJYuu8LzwQ0kvIBGHBf4AP+LAIVSuywNrI\
        GdqmAxHJeJCdsVC1VrvDFXuxmPrMONoHPVrF/qCr5HysNXqfNbqw/kANwwCvAhELVguzAkGS2kEAKRB5JRHGZLvLZizBFByy4YsPukANGzzMA5G1fRDC/tCx5NmknLzRJ5zHWDueW8zL0izI1Az+t8osuLrAor4g\
        EN2Qw+Fsx+ScndSwCzi9C8vgD6tKxgRBD+4sEK9wuQLtD7e6rsJwqwP8t73KoonKuwFNEK3QBSWAJgOAACngBnp4EgqNwj7Ny75MwSfbB/psDkXqorSLBwRRDw08ELb6B/r5tXH8qnrw0iK9DFWas/4ADix60sGc\
        0kUaprq6oUPdB0fqCsH7vUNM05KMEPQwnn/wCQ+KD7bA0ZxsDETKv4bMogacxn9QD/QQ2shcp33AD/7QuiIq0C7Lopt7E109xgzdyw5tqR2rB68QC0Ja0Wods1prC6LctXsbp4MQC6+gtWTbqtxworHgn7T6vCht\
        qStqta/+8ArxHJ9K3AeacLulu9MzPc6NjRDdcKL3mdke7A+PzaKFzLpCurChm94D0bsGjA+kPd5F+qE4Ib14IMZ+kK2WirKFjLC7aw6YoKIpigmAKqAA6qp/PKTpTQ/7TQ3XmqKGUKn7zZ7ES+CqIJ5pKqB1G70z\
        a6ApeghYmrRVqweD4AuDgAetGg/2Ca8I7sMHoQ6LbLV4cAijMLECkeJ4EKawe581Gwv2adeYep/xSQ+hQOApOghiPBXrMAy+gA4PIQ6/IAxrLBDr8AupihD1MAzFUOWAu6KVe8UjThDm8Av6zBHZ8Au7AMAn0Qy/\
        8At6jRnVXMIw4hFzXuchAbDriueKfN7nfv7ngB7ogj7ohF7ohn7oiJ7oir7ojN7ojv7okB7pkj7plF7pln7pmJ7pmr7pnN7pnv7poB7qoj7qpF7qpn7qqJ7qqr7qrN7qrv7qsB7rsj7rtF7rtn7ruJ7rur7rvN7r\
        vv7rwB7swj7sxF7sxn7syJ7syr7szN7szv7s0B7t0j7t1F7tSREQADs=
        """
        return photo
        
class StdoutToLabel(Tkinter.Label):
    """
    The special widget that listens to sys.stdout and updates its
    label accordingly. This widget also owns the progressbar it is
    placed on and updates it every time sys.stdout is called.
    Use stop() to stop retrieving from sys.stdout.
    Credit: http://stackoverflow.com/questions/2914603/
    """

    def __init__(self, widget, progressbar=None, width='default'):
        Tkinter.Label.__init__(self)
        self.defstdout = sys.stdout
        self.widget = widget
        self.progressbar = progressbar
        
        if width == 'default':
            try:
                self.width = widget.cget('width')
            except:
                self.width = None
        else:
            self.width = width   

    def flush(self):
        """
        Frame sys.stdout's flush method.
        """
        self.defstdout.flush()

    def write(self, string, end=None):
        """
        Frame sys.stdout's write method. Puts input
        strings in the widget.
        """
        self.defstdout.write(string)
        if string == "\n":
            pass
        elif len(string) > self.width and self.width is not None and self.width > 0:
            self.widget.config(text="See log file for details")
        elif string is not None:

            last_char = string[-1]

            if last_char != '\n' and string.startswith('\r'):
                number = string.lstrip("\r").rstrip("%")
                number = number.split(".")[0]
                number = int(number)
                self.progressbar["value"] = number                
                
            elif last_char != '\n':
                self.widget.config(text=string)

            elif last_char == '\n' and string.startswith('\r'):
                number = string.lstrip("\r").rstrip("%")
                number = number.split(".")[0]
                number = int(number)
                self.progressbar["value"] = number
                
            else:
                self.widget.config(text=string[:-1])
                
        self.update()
        self.progressbar.update()

    def start(self):
        """
        Starts retrieving.
        """
        sys.stdout = self

    def stop(self):
        """
        Stops retrieving.
        """
        sys.stdout = self.defstdout

    @property
    def errors(self):
        return self.defstdout.errors

    @property
    def encoding(self):
        return self.defstdout.encoding
        
def main():
    root = ScoaryGUI(None)
    root.title("Scoary")
    root.geometry("800x400")
    root.resizable(0,0)
    root.mainloop()
    
if __name__ == "__main__":
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Script for associating Roary output with phenotypic trait (+/-)
# Idea and implementation: Ola Brynildsrud (olbb@fhi.no)

# Null hypothesis: Gene is equally distributed in Trait+ and Trait- isolates

import argparse
import sys
import csv
import time
import random
from multiprocessing import Pool, Value
from scipy import stats as ss
from scipy import spatial
from .citation import citation
from .classes import Matrix
from .classes import QuadTree
from .classes import PhyloTree
import scoary

import os
from pkg_resources import resource_string, resource_filename

SCOARY_VERSION = scoary.__version__

# Python 2/3 annoyances
try:
    xrange
except NameError:
    xrange = range
    

def main(**kwargs):
    """
    The main function of Scoary.
    """
    # If main has been ran from the GUI, then args already exists
    if len(kwargs) == 0:
        args, cutoffs = ScoaryArgumentParser()
    else:
        args = kwargs["args"]
        cutoffs = kwargs["cutoffs"]
        sys.stdout = kwargs["statusbar"]
    
    if args.citation:
        sys.exit(citation())
   
    if args.test:
        args.correction = ['I','EPW']
        args.delimiter = ','
        args.genes = os.path.join(resource_filename(__name__, 'exampledata'), 'Gene_presence_absence.csv')
        args.max_hits = None
        args.newicktree = None
        args.outdir = './'
        args.permute = 0
        args.p_value_cutoff = [0.05,0.05]
        args.restrict_to = None
        args.start_col = 15
        args.threads = 4
        args.traits = os.path.join(resource_filename(__name__, 'exampledata'), 'Tetracycline_resistance.csv')
        args.upgma_tree = True
        args.write_reduced = False
        args.no_time = False
        args.collapse = False
        cutoffs = {"I": 0.05, "EPW": 0.05}
        
    if not args.outdir.endswith("/"):
        args.outdir += "/"
    if args.traits is None or args.genes is None:
        sys.exit("The following arguments are required: -t/--traits, -g/--genes")
    if args.threads <= 0:
        sys.exit("Number of threads must be positive")
    if not os.path.isfile(args.traits):
        sys.exit("Could not find the traits file: %s" % args.traits)
    if not os.path.isdir(args.outdir):
        print("Outdir does not exist, checking for write permissions")
        if os.access(args.outdir, os.W_OK) and os.access(args.outdir, os.X_OK):
            print("Appear to have write permission to outdir")
        else:
            sys.exit("Need to have write (and exec) permission to outdir")
    if not os.path.isfile(args.genes):
        sys.exit("Could not find the gene presence absence file: %s" % args.genes)
    if (args.newicktree is not None) and (not os.path.isfile(args.newicktree)):
        sys.exit("Could not find the custom tree file: %s" % args.newicktree)
    if not all( [(p <= 1.0 and p > 0.0) for p in args.p_value_cutoff] ):
        sys.exit("P must be between 0.0 and 1.0 or exactly 1.0")
    if (len(args.delimiter) > 1):
        sys.exit("Delimiter must be a single character string. There is no support for tab.")
    if (len(args.p_value_cutoff) != len(args.correction)) and (len(args.p_value_cutoff) != 1):
        sys.exit("You can not use more p-value cutoffs than correction methods. Either provide a single "
        "p-value that will be applied to all correction methods, or provide exactly as many as the number "
        "of correction methods and in corresponding sequence. e.g. -c Individual Pairwise_both -p 0.1 0.05 "
        "will apply an individual p-value cutoff of 0.1 AND a pairwise comparisons p-value cutoff of 0.05.")
    if "P" in cutoffs and args.permute == 0:
        sys.exit("Cannot use empirical p-values in filtration without performing permutations. "
        "Use '--permute X' where X is a number equal to or larger than 10")
    if args.permute < 10 and args.permute != 0:
        sys.exit("The absolute minimum number of permutations is 10 (or 0 to deactivate)")
    elif args.permute > 0:
        try:
            random.shuffle
        except:
            sys.exit("Unable to find random.shuffle. Can not proceed with permutations")
    if "P" in cutoffs:
        if cutoffs["P"] < (1.0/args.permute):
            sys.exit("Permutation cutoff too low for this number of permutations")
    if args.permute > 10000:
        print("Warning: You have set Scoary to do a high number of permutations. This may take a while.")
        
    starttime = time.time()
        
    # Start analysis
    with open(args.genes, "rU") as genes, open(args.traits, "rU") as traits:

        if args.restrict_to is not None:
            allowed_isolates = [isolate
                                for line in
                                open(args.restrict_to,"rU")
                                for isolate in line.rstrip().split(",")]
        else:
            # Despite the confusing name
            # this actually means all isolates are allowed
            # and included in the analysis
            allowed_isolates = None
            if args.write_reduced:
                sys.exit("You cannot use the -w argument without specifying a subset (-r)")
            
        print("Reading gene presence absence file")
        
        genedic_and_matrix = Csv_to_dic_Roary(genes,
                                              args.delimiter,
                                              startcol=int(args.start_col) - 1,
                                              allowed_isolates=allowed_isolates,
                                              writereducedset=args.write_reduced,
                                              no_time=args.no_time)
        genedic = genedic_and_matrix["Roarydic"]
        zeroonesmatrix = genedic_and_matrix["Zero_ones_matrix"]
        strains = genedic_and_matrix["Strains"]
        
        if (args.newicktree) is None:
            print("Creating Hamming distance matrix based on gene presence/absence")
            TDM = CreateTriangularDistanceMatrix(zeroonesmatrix, strains)
            QT = PopulateQuadTreeWithDistances(TDM)
            print("Building UPGMA tree from distance matrix")
            upgmatree = upgma(QT)
        else:
            print("Reading custom tree file")
            from .nwkhandler import ReadTreeFromFile
            upgmatree, members = ReadTreeFromFile(args.newicktree)
            if sorted(strains) != sorted(members):
                sys.exit("ERROR: Please make sure that isolates in your custom tree "
                "match those in your gene presence absence file.")    
        
        print("Reading traits file")
        traitsdic = Csv_to_dic(traits, args.delimiter, allowed_isolates)

        print("Finished loading files into memory.")
        print(filtrationoptions(cutoffs, args.collapse))
        print("Tallying genes and performing statistical analyses")

        RES_and_GTC = Setup_results(genedic, traitsdic, args.collapse)
        RES = RES_and_GTC["Results"]
        GTC = RES_and_GTC["Gene_trait_combinations"]

        if args.upgma_tree:
            StoreUPGMAtreeToFile(upgmatree, args.outdir, no_time=args.no_time)

        StoreResults(RES,
                     args.max_hits,
                     cutoffs,
                     upgmatree, GTC,
                     args.outdir,
                     args.permute,
                     args.threads,
                     no_time=args.no_time,
                     delimiter=args.delimiter)
        print("\nFinished. Checked a total of %d genes for associations to %d trait(s). "
              "Total time used: %d seconds." % (len(genedic),
                                                len(traitsdic),
                                                int(time.time()-starttime)))

    sys.exit(0)


def CreateTriangularDistanceMatrix(zeroonesmatrix, strainnames):
    """
    Converts a raw matrix of 0s and 1s (indicating gene presence and absence,
    rows correspond to genes and columns to allowed strains) to an (upper)
    triangular matrix of pairwise Hamming distances.
    The distance d(i,i) is set to 1 for all i.
    """
    try:
        hamming_distances = list(spatial.distance.pdist(zeroonesmatrix, 'hamming'))
    except TypeError:
        sys.exit("Could not locate scipy.spatial.distance.pdist. Perhaps you have an old version of SciPy installed?")
    nstrains = int((1 + (1 + 8*len(hamming_distances))**0.5)/2)
    TriangularDistanceMatrix = []
    Strain_names = []
    for i in xrange(nstrains):
        add = [1]  # Adding the maximum relative hamming distance to prevent Quadtree algorithm to pick a pair where i = j
        add += hamming_distances[:(nstrains-i-1)]
        hamming_distances = hamming_distances[(nstrains-i-1):]
        TriangularDistanceMatrix.append(add)
        Strain_names.append(strainnames[i])

    return {"matrix": TriangularDistanceMatrix, "names": Strain_names}


def PopulateQuadTreeWithDistances(TDM):
    """
    Takes a triangular distance matrix, creates a quadtree and populates it
    with the hamming distances between isolates.
    First creates a Quadmatrix, so not really optimized.
    """
    Quadmatrix = Matrix(dim=len(TDM["matrix"]))
    for i in xrange(Quadmatrix.dim):
        for j in xrange(i, Quadmatrix.dim):
            try:
                Quadmatrix[i][j] = Quadmatrix[j][i] = TDM["matrix"][i][(j-i)]
            except IndexError:
                sys.exit("There was an error trying to populate the Quadtree "
                         "with pairwise distances. Please report this bug to olbb@fhi.no")
    PopulatedQuadtree = QuadTree(Quadmatrix.dim, names=TDM["names"])
    for i in xrange(Quadmatrix.dim):
        PopulatedQuadtree.insert_row(i, Quadmatrix[i])
    return PopulatedQuadtree

def ReduceSet(genefile, delimiter, startcol=14, allowed_isolates=None, no_time=False):
    csvfile = csv.reader(genefile, skipinitialspace=True, delimiter=delimiter)
    header = next(csvfile)
    allowed_indexes = list(range(startcol))
    for c in xrange(len(header)):
        if header[c] in allowed_isolates:
            allowed_indexes.append(c)
    
    print("Writing gene presence absence file for the reduced set of isolates")
    if no_time:
        reducedfilename = "gene_presence_absence_reduced.csv"
    else:
        reducedfilename = "gene_presence_absence_reduced_" + time.strftime("_%d_%m_%Y_%H%M") + ".csv"
    
    with open(reducedfilename, "w") as csvout:
        wtr = csv.writer(csvout, delimiter = delimiter)
        newheader = [header[a] for a in allowed_indexes]
        wtr.writerow(newheader)
        for r in csvfile:
            wtr.writerow( tuple(r[a] for a in allowed_indexes) )
    print("Finished writing reduced gene presence absence list to file " + reducedfilename)
    return reducedfilename

def Csv_to_dic_Roary(genefile, delimiter, startcol=14, allowed_isolates=None, writereducedset=False,no_time=False):
    """
    Converts a gene presence/absence file into dictionaries
    that are readable by Roary
    """
    r = {}
    if writereducedset:
        file = open(ReduceSet(genefile,delimiter,startcol,allowed_isolates,no_time),"rU")
        csvfile = csv.reader(file, skipinitialspace=True, delimiter=delimiter)
    else:
        csvfile = csv.reader(genefile, skipinitialspace=True, delimiter=delimiter)
    header = next(csvfile)
            
    roaryfile = True

    strains = header[startcol:]
    strain_names_allowed = [val for val in strains if val in allowed_isolates] if allowed_isolates is not None else strains
    zero_ones_matrix = []

    try:
        genecol = header.index("Gene")
        nugcol = header.index("Non-unique Gene name")
        anncol = header.index("Annotation")
    except ValueError:
        print("Warning: Could not properly detect the correct names for all columns in the ROARY table.")
        genecol = 0
        nugcol = 1
        anncol = 2

    for line in csvfile:
        q = line
        try:
            r[q[genecol]] = {"Non-unique Gene name": q[nugcol], "Annotation": q[anncol]} if roaryfile else {}
        except IndexError:
            sys.exit("ERROR: Could not read gene presence absence file. Verify that this file is a proper Roary file "
            "using the specified delimiter (default is ',').")
        # The zero_ones_line represents the presence (1) or absence (0) of a gene. It is used for calculating distances between strains.
        zero_ones_line = []

        for strain in xrange(len(strains)):
            if (allowed_isolates is not None) and strains[strain] not in allowed_isolates:
                continue
            if q[startcol + strain] in ["", "0", "-"]:
                # If the gene is not present, AND The isolate is allowed
                r[q[genecol]][strains[strain]] = 0
                zero_ones_line.append(0)
                # Add a 0 to indicate non-presence
            else:
                # Gene is present if any other value than "", "0" or "-" is in the cell
                r[q[genecol]][strains[strain]] = 1
                zero_ones_line.append(1)
                # Add a 1 to indicate presence of the current gene in this strain

        # Since we are only interested in the differences between strains, no need to append the zero_ones_line if it is all 1's (core gene) or all 0's (not in collection)
        if 1 in zero_ones_line and 0 in zero_ones_line:
            zero_ones_matrix.append(zero_ones_line)

    # Transpose list for distance calculation purposes
    if writereducedset:
        file.close()
    zero_ones_matrix = list(map(list, zip(*zero_ones_matrix)))
    return {"Roarydic": r,
            "Zero_ones_matrix": zero_ones_matrix,
            "Strains": strain_names_allowed}

def Csv_to_dic(csvfile, delimiter, allowed_isolates):
    """
    Converts an input traits file (csv format) to dictionaries readable by Roary
    """
    tab = list(zip(*csv.reader(csvfile, delimiter=delimiter)))
    r = {}
    if len(tab) < 2:
        sys.exit("Please check that your traits file is formatted properly and contain at least one trait")
    for num_trait in xrange(1, len(tab)):
        p = dict(zip(tab[0], tab[num_trait]))
        if "" in p:
            name_trait = p[""]
            del p[""]
        elif "Name" in p:
            name_trait = p["Name"]
            del p["Name"]
        else:
            sys.exit("Make sure the top-left cell in the traits file is either empty or 'Name'. Do not include empty rows")

        # Filter so that only allowed isolates are included
        if allowed_isolates is not None:
            p = {strain: indicator for (strain, indicator) in list(p.items())
                 if strain in allowed_isolates}
        r[name_trait] = p

    return r


def Setup_results(genedic, traitsdic, collapse):
    """
    This is the method that actually does all the counting of genes,
    calculation of p-values and post-test adjustment of p-values.
    The majority of running time is currently spent doing Fisher's exact test.
    The running time is somewhat shortened by storing the p-values of known
    2x2 cell distributions, so that identical Fisher's tests will not have
    to be run.
    """
    # Need to create one results dictionary for each trait

    all_traits = {}
    gene_trait_combinations = {}
    fisher_calculated_values = {}
    for trait in traitsdic:
        all_traits[trait] = {}
        # Also need a list of all p-values to perform stepwise FDR methods
        p_value_list = []
        print("Gene-wise counting and Fisher's exact tests for trait: " + trait)

        # We also need a number of tests variable for each genedic.
        # (The number of tests may not be the same, depending on how many genes are in 0 or all strains.)
        number_of_tests = len(genedic)
        initial_number_of_tests = number_of_tests  # This number needed for status bar

        # Additionally, we need a dictionary to, for each gene
        # hold the gene-trait status (e.g "AB" or "ab" of each strain)
        gene_trait_combinations[trait] = {}
        distributions = {}

        i = 0  # Progress
        for gene in genedic:
            # Status:
            sys.stdout.write("\r{:.2%}".format(float(i)/initial_number_of_tests))
            sys.stdout.flush()
            i += 1  # Progress

            stats = Perform_statistics(traitsdic[trait], genedic[gene])
            stat_table = stats["statistics"]

            num_pos = stat_table["tpgp"] + stat_table["tpgn"]
            num_neg = stat_table["tngp"] + stat_table["tngn"]

            if (stat_table["tpgp"] + stat_table["tngp"]) == 0:
                number_of_tests -= 1  # Remove 1 from the number of tests
                continue  # proceed to the next gene

            if (stat_table["tpgn"] + stat_table["tngn"]) == 0:
                number_of_tests -= 1
                continue

            # IF gene_trait_combination exists already, ie the current gene is
            # perfectly correlated with another gene, collapse the two into a
            # single unit
            
            # Should consider adding a softer correlation required than 100%,
            # i.e. genes that are highly co-distributed should perhaps be merged
            # earlier?
            if stats["hash"] in distributions and collapse:
                # Find out which gene is correlated
                corr_gene = distributions[stats["hash"]]
                # Collapse the two genes
                newgenename = corr_gene + "-" + gene
                distributions[stats["hash"]] = newgenename
                # Remove 1 from the number of tests
                number_of_tests -= 1
                mergedgenes = True
                
                del(gene_trait_combinations[trait][corr_gene])
                gene_trait_combinations[trait][newgenename] = stats["gene_trait"]
            else:
                distributions[stats["hash"]] = gene
                mergedgenes = False
                gene_trait_combinations[trait][gene] = stats["gene_trait"]
  
            obs_table = [[stat_table["tpgp"],
                          stat_table["tpgn"]],
                         [stat_table["tngp"],
                          stat_table["tngn"]]]
            obs_tuple = (stat_table["tpgp"],
                         stat_table["tpgn"],
                         stat_table["tngp"],
                         stat_table["tngn"])
            if obs_tuple in fisher_calculated_values:
                odds_ratio = fisher_calculated_values[obs_tuple][0]
                p_value = fisher_calculated_values[obs_tuple][1]
            else:
                fisher = ss.fisher_exact(obs_table)
                p_value = fisher[1]
                odds_ratio = fisher[0]
                fisher_calculated_values[obs_tuple] = fisher
                 
            if not mergedgenes:
                all_traits[trait][gene] = {
                "NUGN": genedic[gene]["Non-unique Gene name"],
                "Annotation": genedic[gene]["Annotation"],
                "tpgp": stat_table["tpgp"],
                "tngp": stat_table["tngp"],
                "tpgn": stat_table["tpgn"],
                "tngn": stat_table["tngn"],
                "sens": (float(stat_table["tpgp"]) / num_pos * 100) if num_pos > 0 else 0.0,
                "spes": (float(stat_table["tngn"]) / num_neg * 100) if num_neg > 0 else 0.0,
                "OR": odds_ratio,
                "p_v": p_value}
                p_value_list.append((gene, p_value))
            else:
                del(all_traits[trait][corr_gene])
                all_traits[trait][newgenename] = {
                "NUGN": "",
                "Annotation": "Merged genes",
                "tpgp": stat_table["tpgp"],
                "tngp": stat_table["tngp"],
                "tpgn": stat_table["tpgn"],
                "tngn": stat_table["tngn"],
                "sens": (float(stat_table["tpgp"]) / num_pos * 100) if num_pos > 0 else 0.0,
                "spes": (float(stat_table["tngn"]) / num_neg * 100) if num_neg > 0 else 0.0,
                "OR": odds_ratio,
                "p_v": p_value}
                p_value_list.append((newgenename, p_value))
        
        sys.stdout.write("\r100.00%")
        sys.stdout.flush()
        print("\nAdding p-values adjusted for testing multiple hypotheses")
        
        # Now calculate Benjamini-Hochberg and Bonferroni p-values
        sorted_p_values = sorted(p_value_list, key=lambda x: x[1])  # Sorted list of tuples: (gene, p-value)

        # Find out which p-values are ties
        # Note: Changed from step-down to step-up (from least significant to most significant)
        tie = [sorted_p_values[i-1][1] == sorted_p_values[i][1] for i in xrange(1, len(sorted_p_values))]
        # Initialize dics of corrected p values
        bh_corrected_p_values = {}
        # The least significant gene is entered into the dic
        bh_corrected_p_values[sorted_p_values[len(sorted_p_values)-1][0]] = last_bh = sorted_p_values[len(sorted_p_values)-1][1]

        for (ind, (gene, p)) in reversed(list(enumerate(sorted_p_values[:-1]))):
            bh_corrected_p_values[gene] = min([last_bh, p*number_of_tests/(ind+1.0)]) if not tie[ind] else last_bh
            last_bh = bh_corrected_p_values[gene]

        # Now add values to dictionaries:
        #for gene in genedic:
        for gene in all_traits[trait]:
            all_traits[trait][gene]["B_p"] = min(all_traits[trait][gene]["p_v"] * number_of_tests , 1.0)
            all_traits[trait][gene]["BH_p"] = min(bh_corrected_p_values[gene], 1.0)

    return {"Results": all_traits, "Gene_trait_combinations": gene_trait_combinations}

def Perform_statistics(traits, genes):
    """
    The method that adds the presence/absence status for each trait-status combination
    """
    r = {"tpgp": 0, "tpgn": 0, "tngp": 0, "tngn": 0}  # tpgn = trait positive, gene negative
    gene_trait = {}
    distribution_hash = ""
    for t in traits:  # For each strain
        try:
            if int(traits[t]) == 1 and genes[t] == 1:
                r["tpgp"] += 1
                distribution_hash += "1"
                gene_trait[t] = "AB"
            elif int(traits[t]) == 1 and genes[t] == 0:
                r["tpgn"] += 1
                distribution_hash += "0"
                gene_trait[t] = "aB"
            elif int(traits[t]) == 0 and genes[t] == 1:
                r["tngp"] += 1
                distribution_hash += "1"
                gene_trait[t] = "Ab"
            elif int(traits[t]) == 0 and genes[t] == 0:
                r["tngn"] += 1
                distribution_hash += "0"
                gene_trait[t] = "ab"
            else:
                sys.exit("There was a problem with comparing your traits and gene presence/absence files."
                "Make sure you have formatted the traits file to specification and only use 1s and 0s, and make sure "
                "the Roary file contains empty cells for non-present genes and non-empty text cells for present genes")
        except KeyError:
            print("\nError occured when trying to find " + str(t) + " in the genes file.")
            sys.exit("Make sure strains are named the same in your traits file as in your gene presence/absence file")
    return {"statistics": r, "hash": int(distribution_hash, 2), "gene_trait": gene_trait}

def StoreResults(Results, max_hits, cutoffs, upgmatree, GTC,
                 outdir, permutations, num_threads, no_time=False, delimiter=","):
    """
    A method for storing the results. Calls StoreTraitResult for each trait column in the input file
    """
    for Trait in Results:
        print("\nStoring results: " + Trait)
        StoreTraitResult(Results[Trait], Trait, max_hits, cutoffs, upgmatree, GTC,
                         outdir, permutations, num_threads, no_time, delimiter)

def StoreTraitResult(Trait, Traitname, max_hits, cutoffs, upgmatree, GTC,
                     outdir, permutations, num_threads, no_time=False, delimiter=","):
    """
    The method that actually stores the results. Only accepts results from a single trait at a time
    """
    permutations = int(permutations)
    if num_threads > 1:
        multithreaded = True
    else:
        multithreaded = False
    
    if not no_time:
        fname = outdir + Traitname + time.strftime("_%d_%m_%Y_%H%M") + ".csv"
    else:
        fname = outdir + Traitname + '.results.csv'

    with open(fname, "w") as outfile:
        # Sort genes by p-value.
        sort_instructions = SortResultsAndSetKey(Trait)
        if max_hits is None:
            max_hits = len(Trait)
        num_results = min(max_hits, len(Trait))

        cut_possibilities = {
            "I": "p_v",
            "B": "B_p",
            "BH": "BH_p",
            "PW": "Plowest",
            "EPW": "Pboth",
            "P": "Empirical_p"
        }
        
        columns = ["Gene","Non-unique gene name","Annotation","Number_pos_present_in","Number_neg_present_in",
        "Number_pos_not_present_in","Number_neg_not_present_in","Sensitivity","Specificity","Odds_ratio","Naive_p","Bonferroni_p",
        "Benjamini_H_p","Max_Pairwise_comparisons","Max_supporting_pairs","Max_opposing_pairs","Best_pairwise_comp_p","Worst_pairwise_comp_p"]
        
        if permutations >= 10:
            columns.append("Empirical_p")
        
        outfile.write(delimiter.join('"' + c +'"' for c in columns) + "\n")
        
        if permutations >= 10:
            print("Calculating max number of contrasting pairs for each significant gene and performing %s permutations" % str(permutations))
        else:
            print("Calculating max number of contrasting pairs for each nominally significant gene")
      
        # Use threads to resolve pairwise comparisons and permutations
        # Each subprocess will be responsible for a domain of the results list
        
        if multithreaded:
            domains = [list(xrange(tnum, num_results, num_threads)) for tnum in xrange(num_threads)]
            # The Progress variable holds the total progress status of the subprocesses. It is locked, but can be written to by all subprocs
            Progress = Value("h",0)
            pool = Pool(processes = num_threads,initializer=initProcess,initargs=(Progress,))
            argumentlist = {"si":sort_instructions,"tree":upgmatree,"GTC": GTC[Traitname],"cutoffs": cutoffs,"cp": cut_possibilities,"perm":permutations,"Trait":Trait, "Threaded": True}
            all_args = [ (domains[x], dict(argumentlist)) for x in xrange(len(domains)) ] # Need to make a copy of dict to prevent then from pointing to the same object
        
            Threadresults = pool.imap(PairWiseComparisons,all_args)

            pool.close()
            pool.join()
        else:
            domains = list(xrange(num_results))
            argumentlist = {"si":sort_instructions,"tree":upgmatree,"GTC": GTC[Traitname],"cutoffs": cutoffs,"cp": cut_possibilities,"perm":permutations,"Trait":Trait, "Threaded": False}
            all_args = (domains, argumentlist)
            Threadresults = [PairWiseComparisons(all_args)]
        sys.stdout.write("\r100.00%")
        sys.stdout.flush()

        # Wait for each thread to finish and weave results from all threads

        # Finally, create a filteredresults that merges results data from Trait[currentgene] with Threadresult[currentgene]
        Filteredresults = {}
        Threadresults = list(Threadresults)

        for thread in xrange(num_threads):
            for currentgene in Threadresults[thread]:
                Filteredresults[currentgene] = Threadresults[thread][currentgene]
                Filteredresults[currentgene].update(Trait[currentgene])
       
        # Only sort by pairwise values if they are the only present correction methods
        if ("I" in cutoffs) or ("B" in cutoffs) or ("BH" in cutoffs):
            sort_instructions = SortResultsAndSetKey(Filteredresults)
        elif ("PW" in cutoffs) or ("EPW" in cutoffs):
            sort_instructions = SortResultsAndSetKeyPairwise(Filteredresults,cutoffs)
        elif ("P" in cutoffs):
            sort_instructions = SortResultsAndSetKey(Filteredresults,key="Empirical_p")
        else:
            print("No filtration applied")
        
        print("\nStoring results to file")       
        num_filteredresults = min(num_results, len(Filteredresults))
        for x in xrange(num_filteredresults):
            currentgene = sort_instructions[x]
            
            # Final check that the currentgene passes all filters
            if all( [Filteredresults[currentgene][cut_possibilities[method]] <= cutoffs[method] for method in cutoffs] ):
                outrow = [currentgene, str(Filteredresults[currentgene]["NUGN"]), str(Filteredresults[currentgene]["Annotation"]), str(Filteredresults[currentgene]["tpgp"]),
                str(Filteredresults[currentgene]["tngp"]), str(Filteredresults[currentgene]["tpgn"]), str(Filteredresults[currentgene]["tngn"]), str(Filteredresults[currentgene]["sens"]),
                str(Filteredresults[currentgene]["spes"]), str(Filteredresults[currentgene]["OR"]), str(Filteredresults[currentgene]["p_v"]), str(Filteredresults[currentgene]["B_p"]),
                str(Filteredresults[currentgene]["BH_p"]), str(Filteredresults[currentgene]["max_total_pairs"]), str(Filteredresults[currentgene]["max_propairs"]), str(Filteredresults[currentgene]["max_antipairs"]),
                str(Filteredresults[currentgene]["Pbest"]),str(Filteredresults[currentgene]["Pworst"])]
            
                # If permutations have been performed, print empirical p-values as well
                if permutations >= 10:
                    outrow.append(str(Filteredresults[currentgene]["Empirical_p"]))
                        
                outfile.write(delimiter.join('"' + c + '"' for c in outrow) + "\n")

def initProcess(share):
    scoary.Progress = share
            
def PairWiseComparisons(nestedlist):
    """
    A threaded version of the pairwise comparisons correction method. Also calculates permutations.
    """
    domain = nestedlist[0]
    sort_instructions = nestedlist[1]["si"]
    tree = nestedlist[1]["tree"]
    GTC = nestedlist[1]["GTC"]
    cutoffs = nestedlist[1]["cutoffs"]
    cutposs = nestedlist[1]["cp"]
    permutations = nestedlist[1]["perm"]
    Trait = nestedlist[1]["Trait"]
    Threaded = nestedlist[1]["Threaded"]
    
    resultscontainer = {}
    num_tot_res = len(Trait)
    progress = 0.0

    for genenumber in domain:
        if Threaded:
            with scoary.Progress.get_lock():
                scoary.Progress.value += 1
            sys.stdout.write("\r{:.2%}".format(float(scoary.Progress.value)/num_tot_res))
            sys.stdout.flush()
        else:
            progress += 1.0
            sys.stdout.write("\r{:.2%}".format(progress/num_tot_res))
            sys.stdout.flush()
        # Start with lowest p-value, the one which has key 0 in sort_instructions
        currentgene = sort_instructions[genenumber]
        resultscontainer[currentgene] = {}
        
        Max_pairwise_comparisons = ConvertUPGMAtoPhyloTree(tree, GTC[currentgene])

        resultscontainer[currentgene]["max_total_pairs"] = Max_pairwise_comparisons["Total"]
        resultscontainer[currentgene]["max_propairs"] = Max_pairwise_comparisons["Pro"]
        resultscontainer[currentgene]["max_antipairs"] = Max_pairwise_comparisons["Anti"]
        
        # Switch definition of "best" and "worst" if antipairs outnumber propairs
        if Max_pairwise_comparisons["Pro"] >= Max_pairwise_comparisons["Anti"]:
            Pbest = "Pbest"
            Pworst = "Pworst"
        else:
            Pbest = "Pworst"
            Pworst = "Pbest"
        try:
            resultscontainer[currentgene][Pbest] = ss.binom_test(resultscontainer[currentgene]["max_propairs"],
                                                                   resultscontainer[currentgene]["max_total_pairs"],
                                                                   0.5)
            resultscontainer[currentgene][Pworst] = ss.binom_test(resultscontainer[currentgene]["max_total_pairs"]-resultscontainer[currentgene]["max_antipairs"],
                                                                    resultscontainer[currentgene]["max_total_pairs"],
                                                                    0.5)
            resultscontainer[currentgene]["Plowest"] = min(resultscontainer[currentgene]["Pbest"], resultscontainer[currentgene]["Pworst"])
            resultscontainer[currentgene]["Pboth"] = max(resultscontainer[currentgene]["Pbest"], resultscontainer[currentgene]["Pworst"])
        except TypeError:
            sys.exit("There was a problem using scipy.stats.binom_test. Ensure you have a recent distribution of SciPy installed.")
          
        # Loop can break early if filtration includes non-pairwise correction measures, because if these
        # are all above the p then we are not interested in the pairwise results
        
        resultscontainer[currentgene]["p_v"] = Trait[currentgene]["p_v"]
        resultscontainer[currentgene]["B_p"] = Trait[currentgene]["B_p"]
        resultscontainer[currentgene]["BH_p"] = Trait[currentgene]["BH_p"]
        
        if decideifbreak(cutoffs, resultscontainer[currentgene]):
            # Genes that are cut from the break should not be in the results
            del(resultscontainer[currentgene])
            break    

        # This is also the place to add permutations - AFTER it has been verified that the current gene passes all filtration filters
        if permutations >= 10:
            empirical_p_perm = Permute(tree=tree,GTC=GTC[currentgene],permutations=permutations,cutoffs=cutoffs)
            resultscontainer[currentgene]["Empirical_p"] = empirical_p_perm
    
    return resultscontainer
    
def Permute(tree, GTC, permutations, cutoffs):
    """
    A method used to permute the GTC dataset by label-switching a given number of times
    and calculate corresponding pairwise comparisons p-values for each PhyloTree.
    Finally, it returns the percentile of the unpermuted dataset p-value in the set of
    ordered p-values resulting from the permutation.
    """
    proceed = True
    #aborted = False
    # Guard the original GTC against changes by making a copy
    GTCcopy = dict(GTC)
    # r is the number of replicates with a larger test statistic than the unpermuted
    r = 0
    Unpermuted_tree = ConvertUPGMAtoPhyloTree(tree,GTCcopy)
    
    # Find out if we're after propairs or antipairs - which is more common and would
    # thus give the high test statistic?
    if Unpermuted_tree["Pro"] >= Unpermuted_tree["Anti"]:
        Pro = "Pro"
    else:
        Pro = "Anti"
    Unpermuted_estimator = float(Unpermuted_tree[Pro]) / Unpermuted_tree["Total"]

    if permutations < 10:
        print("Number of permutations too few. The absolute minimum is 10.")
        proceed = False

    if proceed:
        for i in xrange(permutations):
            # Make a permutation using random.shuffle
            PermutedGTC = PermuteGTC(GTCcopy)
            # Send new set to phylotree
            NewPhyloTree = ConvertUPGMAtoPhyloTree(tree, PermutedGTC)
            if float(NewPhyloTree[Pro]) / NewPhyloTree["Total"] >= Unpermuted_estimator:
                r += 1

            # Check how many estimators are higher than the unpermuted
            # If, after more than 30 iterations, r indicates a p > 0.1, abort
            if i >= 30:
                if (1 - ss.binom.cdf(r,i,0.1)) < 0.05:
                    emp_p = (r+1.0)/(i+1.0)
                    break
        else:
            emp_p = (r+1.0)/(permutations+1.0)
    else:
        return None
    
    return emp_p

def PermuteGTC(GTC):
    """
    Returns a permutation of the gene-trait combination dic where trait labels have
    been swapped
    """
    # Set of trait labels to distribute
    trait_labels = [s[-1] for s in GTC.values()]
    # Shuffle list of labels
    random.shuffle(trait_labels)
    for isolate in GTC:
        # Assign the latter character of GTC to isolates but keep the gene status intact
        GTC[isolate] = str(GTC[isolate][0] + trait_labels.pop())
    return GTC
    
def FindClosestIndex(sortedlist, value, index=None):
    """
    Takes a sorted list and a value and finds which index position the value would have
    if entered into the list. Does this through a binary search
    """
    # METHOD DEPRECATED WHEN P-VALUE STATISTICS NOT NEEDED
    if index is None:
        index = 0
        
    listlen = len(sortedlist)
    if listlen == 0:
        return None
    if listlen == 1:
        if value <= sortedlist[0]:
            return 0
        else:
            return 1
    # Bisect list
    middleindex = listlen // 2
    middlevalue = sortedlist[middleindex]
    if value == middlevalue:
        # Ties in list. Place value exactly at index
        return middleindex
    elif value < middlevalue:
        # Index in left part of list
        index += FindClosestIndex(sortedlist[:middleindex], value, index)
    else:
        # Index in right part of list
        index += middleindex + FindClosestIndex(sortedlist[middleindex:],value, index)
    
    return index   
            
def SortResultsAndSetKey(genedic,key="p_v"):
    """
    A method for returning a dictionary where genes are sorted by p-value
    """
    return {i: gene for (i, gene) in enumerate(sorted(genedic,
                                                      key=lambda x: genedic[x][key])) }
                                                      
def SortResultsAndSetKeyPairwise(genedic,correctionmethod):
    """
    A method for returning a dictionary where genes are sorted by pairwise comparison p-values
    """
    if "EPW" in correctionmethod:
        return {i: gene for (i, gene) in enumerate(sorted(genedic,
                                                          key=lambda x: genedic[x]["Pboth"])) }

    elif "PW" in correctionmethod:
        return {i: gene for (i, gene) in enumerate(sorted(genedic,
                                                          key=lambda x: genedic[x]["Plowest"])) }
    else:
        sys.exit("Something went wrong when using this set of correction methods. Please report this bug")
                                                     
def upgma(d):
    """
    Returns a UPGMA tree from a QuadTree distance matrix d. Heavily based on
    original implementation by Christian Storm Pedersen.
    """
    n = d.dim
    cluster = [x for x in d.names]
    size = n * [1]

    while n > 1:
        # While there are more than a single cluster left, find clusters i and j with minimum distance
        i, j = d.argmin()

        # Build list of distances from the new cluster to all the other clusters
        new_cluster = [cluster[i], cluster[j]]
        new_size = size[i] + size[j]
        new_dist = []
        for k in xrange(d.dim):
            if cluster[k] is None:
                new_dist.append(1)
            else:
                new_dist.append((d.get_elm(i,k)*size[i] + d.get_elm(j,k)*size[j]) / new_size)

        # Insert new row/col in d
        new_dist[i] = sys.maxsize
        d.insert_row(i, new_dist)
        d.insert_col(i, new_dist)
        d.insert_row(j, d.dim * [sys.maxsize])
        d.insert_col(j, d.dim * [sys.maxsize])

        cluster[i] = new_cluster
        cluster[j] = None
        size[i] = new_size
        size[j] = 0

        n -= 1

    return new_cluster

def ConvertUPGMAtoPhyloTree(tree, GTC):
    """
    A method that converts the upgma tree (in nested list form) to a PhyloTree.
    It also needs the status (AB, Ab, aB or ab) of all strains that are to be
    included in the analysis. As of 1.6.0 this is also called from permutations
    """

    # TRAVERSING TREE: For each binary division - go to left until hit tip. Then go back
    MyPhyloTree = PhyloTree(leftnode=tree[0],
                            rightnode=tree[1],
                            GTC=GTC)

    return {"Total": MyPhyloTree.max_contrasting_pairs,
            "Pro": MyPhyloTree.max_contrasting_propairs,
            "Anti": MyPhyloTree.max_contrasting_antipairs}
            


def StoreUPGMAtreeToFile(upgmatree, outdir, no_time=False):
    """
    A method for printing the UPGMA tree that is built internally from the 
    hamming distances in the gene presence/absence matrix
    """
    if not no_time:
        treefilename = str(outdir + "Tree" + time.strftime("_%d_%m_%Y_%H%M") + ".nwk")
    else:
        treefilename = str(outdir + "Tree.nwk")
    with open(treefilename, "w") as treefile:
        Tree = str(upgmatree)
        Tree = Tree.replace("[", "(")
        Tree = Tree.replace("]", ")")
        treefile.write(Tree + ";")
        print("Wrote the UPGMA tree to file: %s" % treefilename)        

def filtrationoptions(cutoffs, collapse):
    """
    Converts between easy keys (I, B, etc) and long names of filtration options, for printing
    to the runlog
    """
    translation = {"I": "Individual (Naive)", "B": "Bonferroni", "BH":"Benjamini-Hochberg",
                   "PW":"Pairwise comparison (Best)", "EPW": "Pairwise comparison (Entire range)",
                   "P": "Empirical p-value (permutation-based)"}
    filters = [str(translation[k]) + ":    " + str(v) for k,v in cutoffs.items()] 
    filters.append("Collapse genes:    " + str(collapse))
    return "Filtration options: \n" + "\n".join(filters)
    
def decideifbreak(cutoffs, currentgene):
    """
    Decides if the loop should continue or if all filtration thresholds are violated and
    therefore should break.
    """
    if (("I" in cutoffs) or ("B" in cutoffs) or ("BH" in cutoffs)):
        if "I" in cutoffs and currentgene["p_v"] > cutoffs["I"]:
            return True
        elif "B" in cutoffs and currentgene["B_p"] > cutoffs["B"]:
            return True
        elif "BH" in cutoffs and currentgene["BH_p"] > cutoffs["BH"]:
            return True
        else:
            # All p-values are too low. Cannot break yet.
            return False
    else:
        # Can't break early if none of the basic filtrations are used, because there might
        # be other genes downstream that pass pairwise and empirical filtration 
        return False

def ScoaryArgumentParser():
    """
    Function for parsing the arguments separate from the main function.
    Makes it easier for other scripts to run Scoary without artificially
    constructing a command line
    """
    # Parse arguments.
    parser = argparse.ArgumentParser(
            description='Scoary version %s - Screen pan-genome for '
                        'trait-associated genes' % SCOARY_VERSION,
                        epilog='by Ola Brynildsrud (olbb@fhi.no)')
    parser.add_argument('-t', '--traits',
                        help='Input trait table (comma-separated-values). '
                        'Trait presence is indicated by 1, trait absence by 0. '
                        'Assumes strain names in the first column and trait '
                        'names in the first row')
    parser.add_argument('-g', '--genes',
                        help='Input gene presence/absence table '
                        '(comma-separated-values) from ROARY. '
                        'Strain names must be equal to those in the trait '
                        'table')
    parser.add_argument('-o', '--outdir',
                        help='Directory to place output files. Default = .',
                        default='./')
    parser.add_argument('-p', '--p_value_cutoff',
                        help='P-value cut-off / alpha level. For Fishers, '
                        'Bonferronis, and Benjamini-Hochbergs tests, SCOARY '
                        'will not report genes with higher p-values than this. '
                        'For empirical p-values, this is treated as an alpha '
                        'level instead. I.e. 0.02 will filter all genes '
                        'except the lower and upper percentile from this test. '
                        'Run with "-p 1.0" to report all genes. '
                        'Accepts standard form (e.g. 1E-8). '
                        'Provide a single value (applied to all) or exactly as '
                        'many values as correction criteria and in corresponding '
                        'order. (See example under correction). '
                        'Default = 0.05',
                        nargs='+',
                        default=[0.05],
                        type=float)
    parser.add_argument('-c', '--correction',
                        help='Apply the indicated filtration measure. '
                        'I=Individual (naive) p-value. '
                        'B=Bonferroni adjusted p-value. '
                        'BH=Benjamini-Hochberg adjusted p. '
                        'PW=Best (lowest) pairwise comparison. '
                        'EPW=Entire range of pairwise comparison p-values. '
                        'P=Empirical p-value from permutations. '
                        'You can enter as many correction criteria as you would like. '
                        'These will be associated with the p_value_cutoffs you enter. '
                        'For example "-c I EPW -p 0.1 0.05" will apply the following '
                        'cutoffs: Naive p-value must be lower than 0.1 AND the '
                        'entire range of pairwise comparison values are below 0.05 '
                        'for this gene. Note that the empirical p-values should be '
                        'interpreted at both tails. Therefore, running "-c P -p 0.05" '
                        'will apply an alpha of 0.05 to the empirical (permuted) p-'
                        'values, i.e. it will filter everything except the upper and lower 2.5 '
                        'percent of the distribution. '
                        'Default = Individual p-value. (I)',
                        choices=['I',
                                 'B',
                                 'BH',
                                 'PW',
                                 'EPW',
                                 'P'],
                        nargs='*',
                        default=['I'])
    parser.add_argument('-e','--permute',
                        help='Perform N number of permutations of the significant '
                        'results post-analysis. Each permutation will do a label '
                        'switching of the phenotype and a new p-value is calculated '
                        'according to this new dataset. After all N permutations are '
                        'completed, the results are ordered in ascending order, and '
                        ' the percentile of the original result in the permuted p-value '
                        'distribution is reported.',
                        type=int,
                        default=0)
    parser.add_argument('-m', '--max_hits',
                        help='Maximum number of hits to report. SCOARY will '
                        'only report the top max_hits results per trait',
                        type=int)
    parser.add_argument('-r', '--restrict_to',
                        help='Use if you only want to analyze a subset of your'
                        ' strains. Scoary will read the provided '
                        'comma-separated table of strains and restrict '
                        'analyzes to these.')
    parser.add_argument('-w', '--write_reduced',
                        help='Use with -r if you want Scoary to create a new '
                        'gene presence absence file from your reduced set of '
                        'isolates. Note: Columns 1-14 (No. sequences, '
                        'Avg group size nuc etc) in this file do not reflect '
                        'the reduced dataset. These are taken from the full dataset.',
                        default=False,
                        action='store_true')
    parser.add_argument('-s', '--start_col',
                        help='On which column in the gene presence/absence '
                        'file do individual strain info start. Default=15. '
                        '(1-based indexing)',
                        default=15,
                        type=int)
    parser.add_argument('-u', '--upgma_tree',
                        help='This flag will cause Scoary to write the '
                        'calculated UPGMA tree to a newick file',
                        default=False,
                        action='store_true')
    parser.add_argument('-n', '--newicktree',
                        help='Supply a custom tree (Newick format) for phylogenetic '
                        'instead analyses instead of calculating it internally.',
                        default=None)
    parser.add_argument('--delimiter',
                        help='The delimiter between cells in the gene '
                        'presence/absence and trait files, as well as '
                        'the output file. ',
                        default=',',
                        type=str)
    parser.add_argument('--collapse',
                        help='Add this to collapse correlated genes (genes that have '
                        'identical distribution patterns in the sample) into merged units. ',
                        default=False,
                        action='store_true')
    parser.add_argument('--threads',
                        help='Number of threads to use. Default = 1',
                        type=int,
                        default=1)
    parser.add_argument('--no-time',
                        help='Output file in the form TRAIT.results.csv, '
                        'instead of TRAIT_TIMESTAMP.csv. When used with the '
                        '-w argument will output a reduced gene matrix in the '
                        'form gene_presence_absence_reduced.csv rather than '
                        'gene_presence_absence_reduced_TIMESTAMP.csv ',
                        default=False,
                        action='store_true')
    parser.add_argument('--test',
                        help='Run Scoary on the test set in exampledata, '
                        'overriding all other parameters. ',
                        default=False,
                        action='store_true')
    parser.add_argument('--citation',
                        help='Show citation information, and exit. ',
                        default=False,
                        action='store_true')
    parser.add_argument('--version', help='Display Scoary version, and exit.',
                        action='version',
                        version=SCOARY_VERSION)

    args = parser.parse_args()
    
    if len(args.p_value_cutoff) == 1:
        cutoffs = {c : args.p_value_cutoff[0] for c in args.correction}
    else:
        cutoffs = dict(list(zip(args.correction, args.p_value_cutoff)))
        
    return args, cutoffs

if __name__ == '__main__':
    pass
